%Produces Figures 12, 13, 14 of the article at
%https://www.strollswithmydog.com/canon-high-res-gd-lpf-aa/

% Inputs
f = -3:1/100:3;                         %c/p, frequency range of interest
MTF0 = 0.75;                            %c/p, first null of Classic AA
d = 1/2/MTF0;                           %px, displacement of Classic AA
d2 = 0.88;                              %px, displacement 2 of High-res PSF
d1 = d2 * 37.5/120.5;                   %px, displacement 1 of High-res PSF
FF = 1;                                 %lin fill factor square px aperture

% Code
[Fx,Fy] = meshgrid(f);                  %2D frequencies

MTF4 = abs(cos(pi*d*Fx).*cos(pi*d*Fy)); %Equation (1), 4-dot AA

MTF4_1 = abs(cos(pi*d1*Fx).*cos(pi*d1*Fy));
MTF4_2 = 1/2 * abs((cos(pi*d2*Fx) + cos(pi*d2*Fy)));
MTF16 = MTF4_1 .* MTF4_2;               %Equation (2), 16-dot AA

MTFPA = abs(sinc(Fx*FF).*sinc(Fy*FF));  %pixel aperture, square, linear FF

MTF4_PA = MTF4 .* MTFPA;                %combined Classic AA + px aperture
MTF16_PA = MTF16 .* MTFPA;              %combined High-res AA + px aperture

% Figures
horizontal = floor(length(f)/2)+1;
diagonal = logical(diag(ones(1,length(f))));

figure; mesh(Fx,Fy,MTF16_PA); colormap prism                        %Fig.14
xlabel('c/p'); ylabel('c/p'); zlabel('MTF'); title('High-Res AA+PA');
figure; mesh(Fx,Fy,MTF4_PA); colormap prism
xlabel('c/p'); ylabel('c/p'); zlabel('MTF'); title('Classic AA+PA');

figure; plot(f*sqrt(2),MTF16_PA(diagonal)); grid; xlim([0 2]);      %Fig.12
hold on;  plot(f,MTF4_PA(horizontal,:)); xlabel('c/p'); ylabel('MTF')
grid minor; legend(['High-Res Diagonal  d2=' num2str(d2,3) 'px'],...
['Classic   Horizontal   d=' num2str(d,2) 'px'],'Location','ne')
title('Frequency Response of Combined AA + Px Aperture, 45 deg Offset')

figure; plot(f,MTF16_PA(horizontal,:)); grid; xlim([0 2]);          %Fig.13
hold on; plot(f*sqrt(2),MTF4_PA(diagonal)); xlabel('c/p'); ylabel('MTF')
grid minor; legend(['High-Res Horizontal d2=' num2str(d2,3) 'px'],...
['Classic    Diagonal     d=' num2str(d,2) 'px'],'Location','ne')
title('Frequency Response of Combined AA + Px Aperture, 45 deg Offset')

sd = 0.35;                              %std of gaussian reference
plot(f,exp(-f.^2/(2*sd^2)),'--'); legend(...
    ['High-Res Horizontal d2=' num2str(d2,3) 'px'],...
    ['Classic    Diagonal     d=' num2str(d,2) 'px'],...
    ['Gaussian                 std=' num2str(sd,3) 'px'],'Location','ne')